/* 
 * File:    main.c
 * Author:  Brian Kelly
 * Version: 1.0
 * Created: 06th June 2017
 */

#include <pic16f628.h>
#include <xc.h>

#define FOSC_INTOSCIO        0xFFFC
// CONFIG
#pragma config FOSC     = INTOSCIO
#pragma config WDTE     = OFF
#pragma config PWRTE    = OFF
#pragma config MCLRE    = OFF
#pragma config BOREN    = OFF
#pragma config LVP      = OFF
#pragma config CPD      = OFF
#pragma config CP       = OFF

#define _XTAL_FREQ 4000000          // Telling the compiler, that we are using 4MHz


#define LED_A          PORTBbits.RB0           // 0x01 Pin 6
#define LED_B          PORTBbits.RB1           // 0x02 Pin 7
#define LED_C          PORTBbits.RB2           // 0x04 Pin 8
#define LED_D          PORTBbits.RB3           // 0x08 Pin 9
#define LED_E          PORTBbits.RB4           // 0x10 Pin 10
#define LED_F          PORTBbits.RB5           // 0x20 Pin 11
#define LED_G          PORTBbits.RB6           // 0x40 Pin 12
#define LED_DP         PORTBbits.RB7           // 0x80 Pin 13

#define DATA0          PORTAbits.RA0           // 0x01 Pin 17
#define DATA1          PORTAbits.RA1           // 0x02 Pin 18
#define DATA2          PORTAbits.RA2           // 0x04 Pin 1
#define DATA3          PORTAbits.RA3           // 0x08 Pin 2

#define Enable         PORTAbits.RA4           // 0x10 Pin 3
#define Blank          PORTAbits.RA5           // 0x20 Pin 6
#define Test           PORTAbits.RA6           // 0x40 Pin 15
#define Lamp_Test      PORTAbits.RA7           // 0x80 Pin 16

int     DataInput;
int     NewSegValue;
int     OldSegValue;
int     Control;

// Functions
void BlankDisplay(void);
void LampTestDisplay(void);
void FlashDP(int);
int ProcessDataInput(int);


int main(void)
{
    CMCON = 0x07;               // Turn Comparators off and set pins as I/O
            
    TRISB = 0b00000000;         // TRISB output
    PORTB = 0b00000000;         // PORTB low
    TRISA = 0b11111111;         // TRISA Input
    PORTA = 0b00000000;         // PORTA low

    NewSegValue = 0b00000000;
    Control = 0;
    
    while(1)
    {
        DataInput = PORTA;      
        
        // Test for Blank Input.
        //
        if (!(Blank & 0x01))
        {
             // Blank the display and tell system a control line is active
             BlankDisplay();
             Control = 1;
        } 
        else
        {
            // Test for lamp test, 
            if (!(Test & 0x01))
            {
                // Display Test and tell system a control line is active
                LampTestDisplay();
                Control = 1;
            }
            else
            {
                // Clear the control flag.
                Control = 0;
            }
        }
        
        // If no control lines are active then process this section.
        if (Control != 1)
        {
            // Test for Enable line
            if (!(Enable & 0x01))
            {
                NewSegValue = ProcessDataInput(DataInput);                
//                NewSegValue = DataInput;                

            }

            // Write the value to the 7 Seg display
            PORTB = NewSegValue;
        }
    } 
}


void LampTestDisplay(void)
{
    int DisplayValue = 0b11111111; 
    PORTB = DisplayValue; 
}


void BlankDisplay(void)
{
    int DisplayValue = 0b00000000; 
    PORTB = DisplayValue;
}


// In this section we read the 4 data bits and convert them to 7 Seg Hex Values
int ProcessDataInput(int)
{
    int ReturnValue;
 
    // AND the input with the mask 00001111 to clear the upper bits
    DataInput = DataInput & 0b00001111;
    
//    FlashDP(DataInput);
    
    switch (DataInput)
    {
        case 0x0f: 
            ReturnValue = 0b01110001;
            break;
        case 0x0e: 
            ReturnValue = 0b01111001;
            break;
        case 0x0d: 
            ReturnValue = 0b01011110;
            break;                
        case 0x0c: 
            ReturnValue = 0b00111001;
            break; 
        case 0x0b: 
            ReturnValue = 0b01111100;
            break; 
        case 0x0a: 
            ReturnValue = 0b01110111;
            break; 
        case 0x09: 
            ReturnValue = 0b01100111;
            break; 
        case 0x08: 
            ReturnValue = 0b01111111;
            break; 
        case 0x07: 
            ReturnValue = 0b00000111;
            break;
        case 0x06: 
            ReturnValue = 0b01111101;
            break;
        case 0x05: 
            ReturnValue = 0b01101101;
            break;
        case 0x04: 
            ReturnValue = 0b01100110;
            break;
        case 0x03: 
            ReturnValue = 0b01001111;
            break;
        case 0x02: 
            ReturnValue = 0b01011011;
            break;        
        case 0x01: 
            ReturnValue = 0b00000110;
            break;
        case 0x00: 
            ReturnValue = 0b00111111;
            break;        
        default:
            ReturnValue = 0b01001001;
            break;  
    }
    
    return (ReturnValue);
}

void FlashDP(int Number)
{
    for(int i=0;i<Number;i++)
    {
        __delay_ms(1000);
        LED_DP = 0x01;
        
        __delay_ms(1000);
        LED_DP = 0x00;        
    }       
}